#include "timer.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


unsigned long divisor = 0UL;
float	      Hz, KHz, MHz, GHz;


void timer_module_init(void)
{
#if defined __linux__
    FILE *cpuinfo;
    char buffer [256], *ptr;

    if ((cpuinfo = fopen("/proc/cpuinfo", "r")) != 0) {
	while (fgets(buffer, 256, cpuinfo) != 0)
	    if (strncmp("cpu MHz", buffer, 7) == 0 && (ptr = strchr(buffer, ':')) != 0) {
		MHz = atof(ptr + 2);
		GHz = MHz / 1000;
		KHz = MHz * 1000;
		Hz  = MHz * 1000000;
		break;
	    }

	fclose(cpuinfo);
    }
#endif
}


static void print_time(const char *name, unsigned long long time)
{
    static const char *prefixes [] = { "n", "u", "m", " ", "k", 0 };
    const char	      **prefix	   = prefixes;
    float	      dtime	   = time / GHz;

    while (dtime >= 999.5 && prefix[1] != 0)
       	dtime /= 1000, prefix ++;

    printf("%s = %4.3g %ss", name, dtime, *prefix);
}


void timer_print(struct timer *timer, const char *timer_name)
{
    printf("%-25s: ", timer_name);

    if (timer->count > 0)
    {
	unsigned long long avg = timer->total_time / timer->count;

	print_time("avg", avg);
	print_time(", total", timer->total_time);
	printf(", count = %9u\n", timer->count);
    }
    else
	printf("not used\n");
}
